@@ -172,14 +172,14 @@ static gint64 meta_seq;         /* 0 *//* ATOMIC */
 /* TODO: use GLib's once https://gitlab.gnome.org/GNOME/glib/issues/1076 lands */
 #if defined(__GCC_HAVE_SYNC_COMPARE_AND_SWAP_8)
 static inline gint64
-gst_atomic_int64_inc (volatile gint64 * atomic)
+gst_atomic_int64_inc (gint64 * atomic)
 {
   return __sync_fetch_and_add (atomic, 1);
 }
 #elif defined (G_PLATFORM_WIN32)
 #include <windows.h>
 static inline gint64
-gst_atomic_int64_inc (volatile gint64 * atomic)
+gst_atomic_int64_inc (gint64 * atomic)
 {
   return InterlockedExchangeAdd64 (atomic, 1);
 }
@@ -191,7 +191,7 @@ gst_atomic_int64_inc (volatile gint64 * atomic)
 #define NO_64BIT_ATOMIC_INT_FOR_PLATFORM
 G_LOCK_DEFINE_STATIC (meta_seq);
 static inline gint64
-gst_atomic_int64_inc (volatile gint64 * atomic)
+gst_atomic_int64_inc (gint64 * atomic)
 {
   gint64 ret;
 
@@ -1757,7 +1757,7 @@ gst_buffer_resize_range (GstBuffer * buffer, guint idx, gint length,
 /**
  * gst_buffer_map:
  * @buffer: a #GstBuffer.
- * @info: (out): info about the mapping
+ * @info: (out caller-allocates): info about the mapping
  * @flags: flags for the mapping
  *
  * This function fills @info with the #GstMapInfo of all merged memory
@@ -1787,7 +1787,7 @@ gst_buffer_map (GstBuffer * buffer, GstMapInfo * info, GstMapFlags flags)
  * @buffer: a #GstBuffer.
  * @idx: an index
  * @length: a length
- * @info: (out): info about the mapping
+ * @info: (out caller-allocates): info about the mapping
  * @flags: flags for the mapping
  *
  * This function fills @info with the #GstMapInfo of @length merged memory blocks
@@ -2643,7 +2643,7 @@ static gboolean
 _gst_parent_buffer_meta_init (GstParentBufferMeta * parent_meta,
     gpointer params, GstBuffer * buffer)
 {
-  static volatile gsize _init;
+  static gsize _init;
 
   if (g_once_init_enter (&_init)) {
     GST_DEBUG_CATEGORY_INIT (gst_parent_buffer_meta_debug, "parentbuffermeta",
@@ -2662,7 +2662,7 @@ _gst_parent_buffer_meta_init (GstParentBufferMeta * parent_meta,
 GType
 gst_parent_buffer_meta_api_get_type (void)
 {
-  static volatile GType type = 0;
+  static GType type = 0;
   static const gchar *tags[] = { NULL };
 
   if (g_once_init_enter (&type)) {
@@ -2812,7 +2812,7 @@ static gboolean
 _gst_reference_timestamp_meta_init (GstReferenceTimestampMeta * meta,
     gpointer params, GstBuffer * buffer)
 {
-  static volatile gsize _init;
+  static gsize _init;
 
   if (g_once_init_enter (&_init)) {
     GST_DEBUG_CATEGORY_INIT (gst_reference_timestamp_meta_debug,
@@ -2833,7 +2833,7 @@ _gst_reference_timestamp_meta_init (GstReferenceTimestampMeta * meta,
 GType
 gst_reference_timestamp_meta_api_get_type (void)
 {
-  static volatile GType type = 0;
+  static GType type = 0;
   static const gchar *tags[] = { NULL };
 
   if (g_once_init_enter (&type)) {
