@@ -45,7 +45,7 @@
  * 2) application and 3) user order. Whenever an earlier entry is newer, the
  * later entries will be updated. Since 1.8 you can also provide extra paths
  * where to find presets through the GST_PRESET_PATH environment variable.
- * Presets found in those paths will be concidered as "app presets".
+ * Presets found in those paths will be considered as "app presets".
  */
 /* FIXME:
  * - non racyness
@@ -622,7 +622,7 @@ gst_preset_default_get_property_names (GstPreset * preset)
         g_free (props);
       }
 
-      g_object_unref (child);
+      gst_object_unref (child);
     }
   }
   if (!result) {
@@ -1211,7 +1211,7 @@ gst_preset_get_meta (GstPreset * preset, const gchar * name, const gchar * tag,
 
 /**
  * gst_preset_set_app_dir:
- * @app_dir: the application specific preset dir
+ * @app_dir: (type filename): the application specific preset dir
  *
  * Sets an extra directory as an absolute path that should be considered when
  * looking for presets. Any presets in the application dir will shadow the
@@ -1237,7 +1237,7 @@ gst_preset_set_app_dir (const gchar * app_dir)
  * Gets the directory for application specific presets if set by the
  * application.
  *
- * Returns: (nullable): the directory or %NULL, don't free or modify
+ * Returns: (nullable) (type filename): the directory or %NULL, don't free or modify
  * the string
  */
 const gchar *
@@ -1322,7 +1322,7 @@ gst_preset_base_init (gpointer g_class)
 GType
 gst_preset_get_type (void)
 {
-  static volatile gsize type = 0;
+  static gsize type = 0;
 
   if (g_once_init_enter (&type)) {
     GType _type;
