@@ -91,30 +91,33 @@ struct _GstTypeFind {
   gpointer _gst_reserved[GST_PADDING];
 };
 
-GST_EXPORT
+/**
+ * gst_type_find_get_type: (attributes doc.skip=true)
+ */
+GST_API
 GType     gst_type_find_get_type   (void);
 
 /* typefind function interface */
 
-GST_EXPORT
+GST_API
 const guint8 *  gst_type_find_peek       (GstTypeFind   * find,
                                           gint64          offset,
                                           guint           size);
-GST_EXPORT
+GST_API
 void            gst_type_find_suggest    (GstTypeFind   * find,
                                           guint           probability,
                                           GstCaps       * caps);
-GST_EXPORT
+GST_API
 void            gst_type_find_suggest_simple (GstTypeFind * find,
                                               guint         probability,
                                               const char  * media_type,
                                               const char  * fieldname, ...);
-GST_EXPORT
+GST_API
 guint64   gst_type_find_get_length (GstTypeFind   * find);
 
 /* registration interface */
 
-GST_EXPORT
+GST_API
 gboolean  gst_type_find_register   (GstPlugin            * plugin,
                                     const gchar          * name,
                                     guint                  rank,
