@@ -22,6 +22,7 @@
 #define __GST_BASE_TRANSFORM_H__
 
 #include <gst/gst.h>
+#include <gst/base/base-prelude.h>
 
 G_BEGIN_DECLS
 
@@ -122,10 +123,10 @@ struct _GstBaseTransform {
  *                  Subclasses can override this method to check if @caps can be
  *                  handled by the element. The default implementation might not be
  *                  the most optimal way to check this in all cases.
- * @set_caps:       allows the subclass to be notified of the actual caps set.
+ * @set_caps:       Allows the subclass to be notified of the actual caps set.
  * @query:          Optional.
  *                  Handle a requested query. Subclasses that implement this
- *                  should must chain up to the parent if they didn't handle the
+ *                  must chain up to the parent if they didn't handle the
  *                  query
  * @decide_allocation: Setup the allocation parameters for allocating output
  *                    buffers. The passed in query contains the result of the
@@ -152,7 +153,7 @@ struct _GstBaseTransform {
  *                  The default implementation uses get_unit_size and keeps
  *                  the number of units the same.
  * @get_unit_size:  Required if the transform is not in-place.
- *                  get the size in bytes of one unit for the given caps.
+ *                  Get the size in bytes of one unit for the given caps.
  * @start:          Optional.
  *                  Called when the element starts processing.
  *                  Allows opening external resources.
@@ -180,7 +181,7 @@ struct _GstBaseTransform {
  *                 offsets of the buffer.
  * @transform_meta: Optional. Transform the metadata on the input buffer to the
  *                  output buffer. By default this method copies all meta without
- *                  tags. subclasses can implement this method and return %TRUE if
+ *                  tags. Subclasses can implement this method and return %TRUE if
  *                  the metadata is to be copied.
  * @before_transform: Optional.
  *                    This method is called right before the base class will
@@ -198,16 +199,16 @@ struct _GstBaseTransform {
  *                  member variable. If the buffer is dropped due to QoS, it returns
  *                  GST_BASE_TRANSFORM_FLOW_DROPPED. If this input buffer is not
  *                  contiguous with any previous input buffer, then @is_discont
- *                  is set to %TRUE. (Since 1.6)
+ *                  is set to %TRUE. (Since: 1.6)
  * @generate_output: Called after each new input buffer is submitted repeatedly
  *                   until it either generates an error or fails to generate an output
  *                   buffer. The default implementation takes the contents of the
  *                   @queued_buf variable, generates an output buffer if needed
  *                   by calling the class @prepare_output_buffer, and then
  *                   calls either @transform or @transform_ip. Elements that don't
- *                   do 1-to-1 transformations on input to output buffers can either
+ *                   do 1-to-1 transformations of input to output buffers can either
  *                   return GST_BASE_TRANSFORM_FLOW_DROPPED or simply not generate
- *                   an output buffer until they are ready to do so. (Since 1.6)
+ *                   an output buffer until they are ready to do so. (Since: 1.6)
  *
  * Subclasses can override any of the available virtual methods or not, as
  * needed. At minimum either @transform or @transform_ip need to be overridden.
@@ -300,58 +301,59 @@ struct _GstBaseTransformClass {
   gpointer       _gst_reserved[GST_PADDING_LARGE - 2];
 };
 
-GST_EXPORT
+GST_BASE_API
 GType           gst_base_transform_get_type         (void);
 
-GST_EXPORT
+GST_BASE_API
 void		gst_base_transform_set_passthrough  (GstBaseTransform *trans,
 	                                             gboolean passthrough);
-GST_EXPORT
+GST_BASE_API
 gboolean	gst_base_transform_is_passthrough   (GstBaseTransform *trans);
 
-GST_EXPORT
+GST_BASE_API
 void		gst_base_transform_set_in_place     (GstBaseTransform *trans,
 	                                             gboolean in_place);
-GST_EXPORT
+GST_BASE_API
 gboolean	gst_base_transform_is_in_place      (GstBaseTransform *trans);
 
-GST_EXPORT
+GST_BASE_API
 void		gst_base_transform_update_qos       (GstBaseTransform *trans,
 						     gdouble proportion,
 						     GstClockTimeDiff diff,
 						     GstClockTime timestamp);
-GST_EXPORT
+GST_BASE_API
 void		gst_base_transform_set_qos_enabled  (GstBaseTransform *trans,
 		                                     gboolean enabled);
-GST_EXPORT
+GST_BASE_API
 gboolean	gst_base_transform_is_qos_enabled   (GstBaseTransform *trans);
 
-GST_EXPORT
+GST_BASE_API
 void            gst_base_transform_set_gap_aware    (GstBaseTransform *trans,
                                                      gboolean gap_aware);
-GST_EXPORT
+GST_BASE_API
 void            gst_base_transform_set_prefer_passthrough (GstBaseTransform *trans,
                                                            gboolean prefer_passthrough);
-GST_EXPORT
+GST_BASE_API
 GstBufferPool * gst_base_transform_get_buffer_pool  (GstBaseTransform *trans);
 
-GST_EXPORT
+GST_BASE_API
 void            gst_base_transform_get_allocator    (GstBaseTransform *trans,
                                                      GstAllocator **allocator,
                                                      GstAllocationParams *params);
-GST_EXPORT
+GST_BASE_API
 void		gst_base_transform_reconfigure_sink (GstBaseTransform *trans);
 
-GST_EXPORT
+GST_BASE_API
 void		gst_base_transform_reconfigure_src  (GstBaseTransform *trans);
 
-GST_EXPORT
+GST_BASE_API
 gboolean gst_base_transform_update_src_caps (GstBaseTransform *trans,
                                              GstCaps *updated_caps);
 
-#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
+GST_BASE_API
+gboolean gst_base_transform_reconfigure (GstBaseTransform * trans);
+
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstBaseTransform, gst_object_unref)
-#endif
 
 G_END_DECLS
 
