@@ -25,6 +25,7 @@
  */
 /**
  * SECTION:gstnetclientclock
+ * @title: GstNetClientClock
  * @short_description: Special clock that synchronizes to a remote time
  *                     provider.
  * @see_also: #GstClock, #GstNetTimeProvider, #GstPipeline
@@ -60,6 +61,7 @@
 #include "gstnettimepacket.h"
 #include "gstntppacket.h"
 #include "gstnetclientclock.h"
+#include "gstnetutils.h"
 
 #include <gio/gio.h>
 
@@ -104,6 +106,7 @@ G_GNUC_INTERNAL GType gst_net_client_internal_clock_get_type (void);
  * more often than 1/20th second (arbitrarily, to spread observations a little) */
 #define DEFAULT_MINIMUM_UPDATE_INTERVAL (GST_SECOND / 20)
 #define DEFAULT_BASE_TIME       0
+#define DEFAULT_QOS_DSCP        -1
 
 /* Maximum number of clock updates we can skip before updating */
 #define MAX_SKIPPED_UPDATES 5
@@ -120,7 +123,8 @@ enum
   PROP_BUS,
   PROP_BASE_TIME,
   PROP_INTERNAL_CLOCK,
-  PROP_IS_NTP
+  PROP_IS_NTP,
+  PROP_QOS_DSCP
 };
 
 struct _GstNetClientInternalClock
@@ -146,6 +150,7 @@ struct _GstNetClientInternalClock
   gchar *address;
   gint port;
   gboolean is_ntp;
+  gint qos_dscp;
 
   /* Protected by OBJECT_LOCK */
   GList *busses;
@@ -158,6 +163,7 @@ struct _GstNetClientInternalClockClass
 
 #define _do_init \
   GST_DEBUG_CATEGORY_INIT (ncc_debug, "netclock", 0, "Network client clock");
+
 G_DEFINE_TYPE_WITH_CODE (GstNetClientInternalClock,
     gst_net_client_internal_clock, GST_TYPE_SYSTEM_CLOCK, _do_init);
 
@@ -210,6 +216,7 @@ gst_net_client_internal_clock_init (GstNetClientInternalClock * self)
   self->port = DEFAULT_PORT;
   self->address = g_strdup (DEFAULT_ADDRESS);
   self->is_ntp = FALSE;
+  self->qos_dscp = DEFAULT_QOS_DSCP;
 
   gst_clock_set_timeout (GST_CLOCK (self), DEFAULT_TIMEOUT);
 
@@ -634,6 +641,7 @@ gst_net_client_internal_clock_thread (gpointer data)
   GstNetClientInternalClock *self = data;
   GSocket *socket = self->socket;
   GError *err = NULL;
+  gint cur_qos_dscp = DEFAULT_QOS_DSCP;
 
   GST_INFO_OBJECT (self, "net client clock thread running, socket=%p", socket);
 
@@ -661,9 +669,19 @@ gst_net_client_internal_clock_thread (gpointer data)
         g_clear_error (&err);
         break;
       } else if (err->code == G_IO_ERROR_TIMED_OUT) {
+        gint new_qos_dscp;
+
         /* timed out, let's send another packet */
         GST_DEBUG_OBJECT (self, "timed out");
 
+        /* before next sending check if need to change QoS */
+        new_qos_dscp = self->qos_dscp;
+        if (cur_qos_dscp != new_qos_dscp &&
+            gst_net_utils_set_socket_tos (socket, new_qos_dscp)) {
+          GST_DEBUG_OBJECT (self, "changed QoS DSCP to: %d", new_qos_dscp);
+          cur_qos_dscp = new_qos_dscp;
+        }
+
         if (self->is_ntp) {
           GstNtpPacket *packet;
 
@@ -950,9 +968,6 @@ gst_net_client_internal_clock_stop (GstNetClientInternalClock * self)
   GST_INFO_OBJECT (self, "stopped");
 }
 
-#define GST_NET_CLIENT_CLOCK_GET_PRIVATE(obj)  \
-  (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GST_TYPE_NET_CLIENT_CLOCK, GstNetClientClockPrivate))
-
 struct _GstNetClientClockPrivate
 {
   GstClock *internal_clock;
@@ -964,6 +979,7 @@ struct _GstNetClientClockPrivate
 
   gchar *address;
   gint port;
+  gint qos_dscp;
 
   GstBus *bus;
 
@@ -972,7 +988,8 @@ struct _GstNetClientClockPrivate
   gulong synced_id;
 };
 
-G_DEFINE_TYPE (GstNetClientClock, gst_net_client_clock, GST_TYPE_SYSTEM_CLOCK);
+G_DEFINE_TYPE_WITH_PRIVATE (GstNetClientClock, gst_net_client_clock,
+    GST_TYPE_SYSTEM_CLOCK);
 
 static void gst_net_client_clock_finalize (GObject * object);
 static void gst_net_client_clock_set_property (GObject * object, guint prop_id,
@@ -992,8 +1009,6 @@ gst_net_client_clock_class_init (GstNetClientClockClass * klass)
   gobject_class = G_OBJECT_CLASS (klass);
   clock_class = GST_CLOCK_CLASS (klass);
 
-  g_type_class_add_private (klass, sizeof (GstNetClientClockPrivate));
-
   gobject_class->finalize = gst_net_client_clock_finalize;
   gobject_class->get_property = gst_net_client_clock_get_property;
   gobject_class->set_property = gst_net_client_clock_set_property;
@@ -1051,6 +1066,12 @@ gst_net_client_clock_class_init (GstNetClientClockClass * klass)
           "Internal clock that directly slaved to the remote clock",
           GST_TYPE_CLOCK, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
+  g_object_class_install_property (gobject_class, PROP_QOS_DSCP,
+      g_param_spec_int ("qos-dscp", "QoS diff srv code point",
+          "Quality of Service, differentiated services code point (-1 default)",
+          -1, 63, DEFAULT_QOS_DSCP,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   clock_class->get_internal_time = gst_net_client_clock_get_internal_time;
 }
 
@@ -1060,13 +1081,14 @@ gst_net_client_clock_init (GstNetClientClock * self)
   GstNetClientClockPrivate *priv;
   GstClock *clock;
 
-  self->priv = priv = GST_NET_CLIENT_CLOCK_GET_PRIVATE (self);
+  self->priv = priv = gst_net_client_clock_get_instance_private (self);
 
   GST_OBJECT_FLAG_SET (self, GST_CLOCK_FLAG_CAN_SET_MASTER);
   GST_OBJECT_FLAG_SET (self, GST_CLOCK_FLAG_NEEDS_STARTUP_SYNC);
 
   priv->port = DEFAULT_PORT;
   priv->address = g_strdup (DEFAULT_ADDRESS);
+  priv->qos_dscp = DEFAULT_QOS_DSCP;
 
   priv->roundtrip_limit = DEFAULT_ROUNDTRIP_LIMIT;
   priv->minimum_update_interval = DEFAULT_MINIMUM_UPDATE_INTERVAL;
@@ -1083,6 +1105,7 @@ update_clock_cache (ClockCache * cache)
 {
   GstClockTime roundtrip_limit = 0, minimum_update_interval = 0;
   GList *l, *busses = NULL;
+  gint qos_dscp = DEFAULT_QOS_DSCP;
 
   GST_OBJECT_LOCK (cache->clock);
   g_list_free_full (GST_NET_CLIENT_INTERNAL_CLOCK (cache->clock)->busses,
@@ -1104,12 +1127,15 @@ update_clock_cache (ClockCache * cache)
     else
       minimum_update_interval =
           MIN (minimum_update_interval, clock->priv->minimum_update_interval);
+
+    qos_dscp = MAX (qos_dscp, clock->priv->qos_dscp);
   }
   GST_NET_CLIENT_INTERNAL_CLOCK (cache->clock)->busses = busses;
   GST_NET_CLIENT_INTERNAL_CLOCK (cache->clock)->roundtrip_limit =
       roundtrip_limit;
   GST_NET_CLIENT_INTERNAL_CLOCK (cache->clock)->minimum_update_interval =
       minimum_update_interval;
+  GST_NET_CLIENT_INTERNAL_CLOCK (cache->clock)->qos_dscp = qos_dscp;
 
   GST_OBJECT_UNLOCK (cache->clock);
 }
@@ -1227,6 +1253,12 @@ gst_net_client_clock_set_property (GObject * object, guint prop_id,
       gst_object_unref (clock);
       break;
     }
+    case PROP_QOS_DSCP:
+      GST_OBJECT_LOCK (self);
+      self->priv->qos_dscp = g_value_get_int (value);
+      GST_OBJECT_UNLOCK (self);
+      update = TRUE;
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1283,6 +1315,11 @@ gst_net_client_clock_get_property (GObject * object, guint prop_id,
     case PROP_INTERNAL_CLOCK:
       g_value_set_object (value, self->priv->internal_clock);
       break;
+    case PROP_QOS_DSCP:
+      GST_OBJECT_LOCK (self);
+      g_value_set_int (value, self->priv->qos_dscp);
+      GST_OBJECT_UNLOCK (self);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1331,6 +1368,7 @@ gst_net_client_clock_constructed (GObject * object)
         g_object_new (GST_TYPE_NET_CLIENT_INTERNAL_CLOCK, "address",
         self->priv->address, "port", self->priv->port, "is-ntp",
         self->priv->is_ntp, NULL);
+    gst_object_ref_sink (cache->clock);
     clocks = g_list_prepend (clocks, cache);
 
     /* Not actually leaked but is cached for a while before being disposed,
@@ -1377,11 +1415,11 @@ gst_net_client_clock_get_internal_time (GstClock * clock)
  * @remote_port: the port of the remote clock provider
  * @base_time: initial time of the clock
  *
- * Create a new #GstNetClientInternalClock that will report the time
- * provided by the #GstNetTimeProvider on @remote_address and 
+ * Create a new #GstNetClientClock that will report the time
+ * provided by the #GstNetTimeProvider on @remote_address and
  * @remote_port.
  *
- * Returns: a new #GstClock that receives a time from the remote
+ * Returns: (transfer full): a new #GstClock that receives a time from the remote
  * clock.
  */
 GstClock *
@@ -1395,8 +1433,12 @@ gst_net_client_clock_new (const gchar * name, const gchar * remote_address,
   g_return_val_if_fail (remote_port <= G_MAXUINT16, NULL);
   g_return_val_if_fail (base_time != GST_CLOCK_TIME_NONE, NULL);
 
-  ret = g_object_new (GST_TYPE_NET_CLIENT_CLOCK, "address", remote_address,
-      "port", remote_port, "base-time", base_time, NULL);
+  ret =
+      g_object_new (GST_TYPE_NET_CLIENT_CLOCK, "name", name, "address",
+      remote_address, "port", remote_port, "base-time", base_time, NULL);
+
+  /* Clear floating flag */
+  gst_object_ref_sink (ret);
 
   return ret;
 }
@@ -1424,7 +1466,7 @@ gst_ntp_clock_init (GstNtpClock * self)
  * Create a new #GstNtpClock that will report the time provided by
  * the NTPv4 server on @remote_address and @remote_port.
  *
- * Returns: a new #GstClock that receives a time from the remote
+ * Returns: (transfer full): a new #GstClock that receives a time from the remote
  * clock.
  *
  * Since: 1.6
@@ -1440,8 +1482,11 @@ gst_ntp_clock_new (const gchar * name, const gchar * remote_address,
   g_return_val_if_fail (remote_port <= G_MAXUINT16, NULL);
   g_return_val_if_fail (base_time != GST_CLOCK_TIME_NONE, NULL);
 
-  ret = g_object_new (GST_TYPE_NTP_CLOCK, "address", remote_address,
+  ret =
+      g_object_new (GST_TYPE_NTP_CLOCK, "name", name, "address", remote_address,
       "port", remote_port, "base-time", base_time, NULL);
 
+  gst_object_ref_sink (ret);
+
   return ret;
 }
