@@ -17,12 +17,17 @@
  * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <gst/check/gstcheck.h>
 
 static GstBus *test_bus = NULL;
 static GMainLoop *main_loop;
 
+static GType foo_device_get_type (void);
+
 #define NUM_MESSAGES 1000
 #define NUM_THREADS 10
 
@@ -186,7 +191,7 @@ GST_START_TEST (test_watch)
   fail_unless_equals_int (num_eos, 10);
   fail_unless_equals_int (num_app, 10);
 
-  g_source_remove (id);
+  fail_unless (gst_bus_remove_watch (test_bus));
   g_main_loop_unref (main_loop);
 
   gst_object_unref ((GstObject *) test_bus);
@@ -284,6 +289,31 @@ GST_START_TEST (test_add_watch_with_custom_context)
 
 GST_END_TEST;
 
+static gboolean
+dummy_bus_func (GstBus * bus, GstMessage * msg, gpointer user_data)
+{
+  return TRUE;
+}
+
+GST_START_TEST (test_remove_watch)
+{
+  test_bus = gst_bus_new ();
+
+  /* removing a non-existing watch should fail */
+  fail_if (gst_bus_remove_watch (test_bus));
+
+  gst_bus_add_watch (test_bus, dummy_bus_func, NULL);
+
+  fail_unless (gst_bus_remove_watch (test_bus));
+
+  /* now it should fail to remove the watch again */
+  fail_if (gst_bus_remove_watch (test_bus));
+
+  gst_object_unref (test_bus);
+}
+
+GST_END_TEST;
+
 static gint messages_seen;
 
 static void
@@ -295,7 +325,7 @@ message_func (GstBus * bus, GstMessage * message, gpointer data)
 }
 
 static void
-send_5app_1el_1err_2app_messages (guint interval_usecs)
+send_5app_1el_1err_2app_1eos_messages (guint interval_usecs)
 {
   GstMessage *m;
   GstStructure *s;
@@ -328,8 +358,36 @@ send_5app_1el_1err_2app_messages (guint interval_usecs)
     gst_bus_post (test_bus, m);
     g_usleep (interval_usecs);
   }
+  for (i = 0; i < 1; i++) {
+    m = gst_message_new_eos (NULL);
+    GST_LOG ("posting EOS message");
+    gst_bus_post (test_bus, m);
+    g_usleep (interval_usecs);
+  }
 }
 
+static void
+send_extended_messages (guint interval_usecs)
+{
+  GstMessage *msg;
+  GstDevice *device;
+
+  device = g_object_new (foo_device_get_type (), NULL);
+
+  msg = gst_message_new_device_added (NULL, device);
+  GST_LOG ("posting device-added message");
+  gst_bus_post (test_bus, msg);
+  g_usleep (interval_usecs);
+
+  msg = gst_message_new_device_removed (NULL, device);
+  GST_LOG ("posting device-removed message");
+  gst_bus_post (test_bus, msg);
+  g_usleep (interval_usecs);
+
+  gst_object_unref (device);
+}
+
+
 static void
 send_10_app_messages (void)
 {
@@ -371,6 +429,94 @@ GST_START_TEST (test_watch_with_poll)
 
 GST_END_TEST;
 
+static gint check_messages_seen;
+
+static void
+send_check_messages (guint index)
+{
+  GstMessage *m;
+  GstStructure *s;
+
+  s = gst_structure_new ("test_message", "index", G_TYPE_INT, index, NULL);
+  m = gst_message_new_application (NULL, s);
+  gst_bus_post (test_bus, m);
+}
+
+static void
+message_check_func (GstBus * bus, GstMessage * message, gpointer data)
+{
+  const GstStructure *s;
+  int index;
+
+  g_return_if_fail (GST_MESSAGE_TYPE (message) == GST_MESSAGE_APPLICATION);
+
+  s = gst_message_get_structure (message);
+  if (!gst_structure_get_int (s, "index", &index))
+    g_critical ("Invalid message");
+
+  check_messages_seen |= 1 << index;
+}
+
+/* test that removing and adding the signal watch again works */
+GST_START_TEST (test_watch_twice)
+{
+  GMainContext *ctx;
+  guint expect = 0;
+
+  test_bus = gst_bus_new ();
+  ctx = g_main_context_new ();
+
+  g_main_context_push_thread_default (ctx);
+
+  send_check_messages (0);
+  expect |= 1 << 0;
+
+  gst_bus_add_signal_watch (test_bus);
+
+  g_signal_connect (test_bus, "message::application",
+      (GCallback) message_check_func, NULL);
+
+  send_check_messages (1);
+  expect |= 1 << 1;
+
+  fail_unless (g_main_context_pending (ctx));
+
+  gst_bus_remove_signal_watch (test_bus);
+
+  send_check_messages (2);
+  expect |= 1 << 2;
+
+  gst_bus_add_signal_watch (test_bus);
+
+  send_check_messages (3);
+  expect |= 1 << 3;
+
+  while (g_main_context_pending (ctx))
+    g_main_context_iteration (ctx, FALSE);
+
+  /* this message will not arrive */
+  send_check_messages (4);
+  expect |= 0 << 4;
+
+  fail_unless (g_main_context_pending (ctx));
+
+  gst_bus_remove_signal_watch (test_bus);
+
+  /* this message should not arrive */
+  send_check_messages (5);
+  expect |= 0 << 5;
+
+  while (g_main_context_pending (ctx))
+    g_main_context_iteration (ctx, FALSE);
+
+  fail_unless (check_messages_seen == expect);
+
+  g_main_context_unref (ctx);
+  gst_object_unref (test_bus);
+}
+
+GST_END_TEST;
+
 /* test that you get the messages with pop. */
 GST_START_TEST (test_timed_pop)
 {
@@ -390,6 +536,29 @@ GST_START_TEST (test_timed_pop)
 
 GST_END_TEST;
 
+typedef struct
+{
+  GstDevice device;
+} FooDevice;
+typedef struct
+{
+  GstDeviceClass device_klass;
+} FooDeviceClass;
+
+G_DEFINE_TYPE (FooDevice, foo_device, GST_TYPE_DEVICE);
+
+static void
+foo_device_class_init (FooDeviceClass * klass)
+{
+  /* nothing to do here */
+}
+
+static void
+foo_device_init (FooDevice * device)
+{
+  /* nothing to do here */
+}
+
 /* test that you get the messages with pop_filtered */
 GST_START_TEST (test_timed_pop_filtered)
 {
@@ -419,7 +588,7 @@ GST_START_TEST (test_timed_pop_filtered)
   msg = gst_bus_timed_pop_filtered (test_bus, 0, GST_MESSAGE_ANY);
   fail_unless (msg == NULL);
 
-  send_5app_1el_1err_2app_messages (0);
+  send_5app_1el_1err_2app_1eos_messages (0);
   msg = gst_bus_timed_pop_filtered (test_bus, 0,
       GST_MESSAGE_ANY ^ GST_MESSAGE_APPLICATION);
   fail_unless (msg != NULL);
@@ -434,6 +603,45 @@ GST_START_TEST (test_timed_pop_filtered)
   fail_unless (msg == NULL);
 
   gst_object_unref (test_bus);
+
+  /* Test extended messages */
+  GST_DEBUG
+      ("Checking extended messages received from gst_bus_timed_pop_filtered");
+  test_bus = gst_bus_new ();
+
+  send_5app_1el_1err_2app_1eos_messages (0);
+  send_extended_messages (0);
+  send_5app_1el_1err_2app_1eos_messages (0);
+  msg = gst_bus_timed_pop_filtered (test_bus, 0, GST_MESSAGE_EXTENDED);
+  fail_unless (msg != NULL);
+  fail_unless_equals_int (GST_MESSAGE_TYPE (msg), GST_MESSAGE_DEVICE_ADDED);
+  gst_message_unref (msg);
+
+  msg = gst_bus_timed_pop_filtered (test_bus, 0, GST_MESSAGE_EXTENDED);
+  fail_unless (msg != NULL);
+  fail_unless_equals_int (GST_MESSAGE_TYPE (msg), GST_MESSAGE_DEVICE_REMOVED);
+  gst_message_unref (msg);
+  gst_object_unref (test_bus);
+
+  /* Now check extended messages don't appear when we don't ask for them */
+  GST_DEBUG
+      ("Checking extended messages *not* received from gst_bus_timed_pop_filtered when not wanted");
+  test_bus = gst_bus_new ();
+
+  send_extended_messages (0);
+  send_5app_1el_1err_2app_1eos_messages (0);
+
+  msg = gst_bus_timed_pop_filtered (test_bus, 0, GST_MESSAGE_ERROR);
+  fail_unless (msg != NULL);
+  fail_unless_equals_int (GST_MESSAGE_TYPE (msg), GST_MESSAGE_ERROR);
+  gst_message_unref (msg);
+
+  msg = gst_bus_timed_pop_filtered (test_bus, 0, GST_MESSAGE_EOS);
+  fail_unless (msg != NULL);
+  fail_unless_equals_int (GST_MESSAGE_TYPE (msg), GST_MESSAGE_EOS);
+  gst_message_unref (msg);
+
+  gst_object_unref (test_bus);
 }
 
 GST_END_TEST;
@@ -442,7 +650,7 @@ static gpointer
 post_delayed_thread (gpointer data)
 {
   THREAD_START ();
-  send_5app_1el_1err_2app_messages (1 * G_USEC_PER_SEC);
+  send_5app_1el_1err_2app_1eos_messages (1 * G_USEC_PER_SEC);
   return NULL;
 }
 
@@ -612,6 +820,86 @@ GST_START_TEST (test_custom_main_context)
 
 GST_END_TEST;
 
+static GstBusSyncReply
+test_async_sync_handler (GstBus * bus, GstMessage * msg, gpointer user_data)
+{
+  GArray *timestamps = user_data;
+  gint64 ts = g_get_monotonic_time () * 1000;   /* microsecs -> nanosecs */
+
+  g_array_append_val (timestamps, ts);
+  GST_INFO ("[msg %p] %" GST_PTR_FORMAT, msg, msg);
+
+  return GST_BUS_ASYNC;
+}
+
+static gpointer
+post_10_app_messages_thread (gpointer data)
+{
+  THREAD_START ();
+  send_10_app_messages ();
+  return NULL;
+}
+
+/* Test GST_BUS_ASYNC actually causes the thread posting the message to
+ * block until the message has been freed. We spawn a thread to post ten
+ * messages. We install a bus sync handler to get the timestamp of each
+ * message as it is being posted, and to return GST_BUS_ASYNC. In the main
+ * thread we sleep a bit after we pop off a message and before we free it.
+ * The posting thread should be blocked while the main thread sleeps, so
+ * we expect the interval as the messages are posted to be roughly the same
+ * as the sleep time in the main thread. g_usleep() is not super-precise, so
+ * we allow for some slack there, we just want to check that the posting
+ * thread was blocked at all really. */
+GST_START_TEST (test_async_message)
+{
+  GArray *timestamps;
+  guint i;
+
+  MAIN_INIT ();
+
+  timestamps = g_array_sized_new (FALSE, FALSE, sizeof (gint64), 10);
+
+  test_bus = gst_bus_new ();
+
+  gst_bus_set_sync_handler (test_bus, test_async_sync_handler, timestamps,
+      NULL);
+
+  MAIN_START_THREAD_FUNCTIONS (1, post_10_app_messages_thread, NULL);
+
+  MAIN_SYNCHRONIZE ();
+
+  for (i = 0; i < 10; i++) {
+    GstMessage *msg;
+
+    GST_LOG ("(%d) waiting for message..", i);
+    msg = gst_bus_timed_pop (test_bus, GST_CLOCK_TIME_NONE);
+    GST_LOG ("(%d) got message, sleeping a bit", i);
+    g_usleep (60 * GST_MSECOND / (GST_SECOND / G_USEC_PER_SEC));
+    GST_LOG ("(%d) about to free message", i);
+    gst_message_unref (msg);
+  }
+
+  for (i = 1; i < 10; i++) {
+    gint64 prev_ts = g_array_index (timestamps, gint64, i - 1);
+    gint64 ts = g_array_index (timestamps, gint64, i);
+    gint64 diff = ts - prev_ts;
+
+    fail_unless (prev_ts < ts);
+    fail_unless (diff >= 20 * GST_MSECOND, "interval between messages being "
+        "posted was just %" G_GINT64_FORMAT "ms", diff / GST_MSECOND);
+  }
+
+  fail_if (gst_bus_have_pending (test_bus), "unexpected messages on bus");
+
+  MAIN_STOP_THREADS ();
+
+  gst_object_unref (test_bus);
+
+  g_array_unref (timestamps);
+}
+
+GST_END_TEST;
+
 static Suite *
 gst_bus_suite (void)
 {
@@ -624,13 +912,16 @@ gst_bus_suite (void)
   tcase_add_test (tc_chain, test_hammer_bus);
   tcase_add_test (tc_chain, test_watch);
   tcase_add_test (tc_chain, test_watch_with_poll);
+  tcase_add_test (tc_chain, test_watch_twice);
   tcase_add_test (tc_chain, test_watch_with_custom_context);
   tcase_add_test (tc_chain, test_add_watch_with_custom_context);
+  tcase_add_test (tc_chain, test_remove_watch);
   tcase_add_test (tc_chain, test_timed_pop);
   tcase_add_test (tc_chain, test_timed_pop_thread);
   tcase_add_test (tc_chain, test_timed_pop_filtered);
   tcase_add_test (tc_chain, test_timed_pop_filtered_with_timeout);
   tcase_add_test (tc_chain, test_custom_main_context);
+  tcase_add_test (tc_chain, test_async_message);
   return s;
 }
 
